{% extends "saas_emails/base.html" %}

{% block title %}Invitation{% endblock %}

{% block content %}
  <p>Hi {{ member.name }},</p>
  <p>{% if inviter.get_short_name %}{{ inviter.get_short_name }}{% else %}{{ inviter.username }}{% endif %}
    has invited you to join the tenant <strong>{{ tenant }}</strong> on {{ site.name }}.</p>
  <p>To accept the invitation and get started:</p>
  <ol>
    <li>Click the button below.</li>
    <li>Follow the instructions to set up your account or log in.</li>
  </ol>
  <div class="button">
    <table role="presentation" border="0" cellpadding="0" cellspacing="0">
      <tbody>
        <tr>
          <td align="left"><a href="{{ invite_link }}">Accept Invitation</a></td>
        </tr>
      </tbody>
    </table>
  </div>
  <hr />
  <p>If you didn't expect this invitation or believe it was sent in error, you can ignore this email.</p>
  <p>Welcome aboard,<br>{{ site.name }}</p>
{% endblock %}
